@@ -1,4 +1,4 @@
-package Affix::Wrap v1.0.7 {
+package Affix::Wrap v1.0.8 {
     use v5.40;
     use feature 'class';
     no warnings 'experimental::class';
@@ -107,7 +107,7 @@ package Affix::Wrap v1.0.7 {
 
             # Case-insensitive lookup (handled by existing code)
             return $type_map->{ lc $t } if defined $type_map->{ lc $t };
-            return $t                   if $t =~ /^[a-z_]\w*$/i;
+            return "'$t'"               if $t =~ /^[a-z_]\w*$/i;
             warn "WARNING: Unknown C type '$t' mapped to Void\n";
             'Void';
         }
@@ -297,17 +297,21 @@ package Affix::Wrap v1.0.7 {
     class    #
         Affix::Wrap::Macro : isa(Affix::Wrap::Entity) {
         field $value : reader : param //= ();
+        method set_value ($v) { $value = $v }
 
         method affix_type {
             $value // return '';
             my $v = $value // '';
             $v =~ s/^\s+|\s+$//g;
             return '' unless length $v;
-            if ( $v =~ /^-?(?:0x[\da-fA-F]+|\d+(?:\.\d+)?)$/ || $v =~ /^".*"$/ || $v =~ /^'.*'$/ ) {
-                return $v;
+            if ( $v =~ /^-?(?:0x[\da-fA-F]+|\d+(?:\.\d+)?)$/ ) {
+                return sprintf 'use constant %s => %s', $self->name, $v;
+            }
+            if ( $v =~ /^".*"$/ || $v =~ /^'.*'$/ ) {
+                return sprintf 'use constant %s => %s', $self->name, $v;
             }
             $v =~ s/'/\\'/g;
-            sprintf 'use constant %s => %s', $self->name, $v;
+            sprintf 'use constant %s => \'%s\'', $self->name, $v;
         }
 
         method affix ( $lib //= (), $pkg //= () ) {
@@ -322,7 +326,7 @@ package Affix::Wrap v1.0.7 {
         }
         } class Affix::Wrap::Variable : isa(Affix::Wrap::Entity) {
         field $type : reader : param;
-        method affix_type { sprintf 'pin my $%s, $lib, %s => %s', $self->name, $self->name, $type->affix_type }
+        method affix_type { sprintf 'pin my $%s, $lib, \'%s\' => %s', $self->name, $self->name, $type->affix_type }
 
         method affix ( $lib, $pkg //= () ) {
             if ($lib) {
@@ -344,7 +348,7 @@ package Affix::Wrap v1.0.7 {
         } class    #
         Affix::Wrap::Typedef : isa(Affix::Wrap::Entity) {
         field $underlying : reader : param;
-        method affix_type { 'typedef ' . $self->name . ' => ' . $underlying->affix_type }
+        method affix_type { 'typedef \'' . $self->name . '\' => ' . $underlying->affix_type }
 
         method affix ( $lib //= (), $pkg //= () ) {
             my $t = $underlying->affix;
@@ -423,9 +427,9 @@ package Affix::Wrap v1.0.7 {
         field $mangled_name : reader : param //= ();
 
         method affix_type {
-            sprintf 'affix $lib, %s => [%s], %s;',
-                ( $self->mangled_name ne $self->name ? ( sprintf q[[%s => '%s']], $self->mangled_name, $self->name ) : $self->name ),
-                join( ', ', @{ $self->affix_args } ), $self->affix_ret;
+            sprintf 'affix $lib, %s => [%s], %s',
+                ( $self->mangled_name ne $self->name ? ( sprintf q[[%s => '%s']], $self->mangled_name, $self->name ) :
+                    ( sprintf q['%s'], $self->name ) ), join( ', ', @{ $self->affix_args } ), $self->affix_ret;
         }
 
         method affix ( $lib, $pkg //= () ) {
@@ -580,7 +584,6 @@ package Affix::Wrap v1.0.7 {
             return 0 unless defined $f && length $f;
             return 0 if $f =~ m{^/usr/(include|lib|share|local/include)};
             return 0 if $f =~ m{^/System/Library};
-            return 0 if $f =~ m{(Program Files|Strawberry|MinGW|Windows|cygwin|msys)}i;
             return 1 if $allowed_files->{$f};
             for my $dir (@$project_dirs) { return 1 if index( $f, $dir ) == 0; }
             return 0;
@@ -919,7 +922,7 @@ package Affix::Wrap v1.0.7 {
                         $_->file eq $td->file                                                    &&
                         ( $_->name eq '(anonymous)' || $_->name eq '' || $_->name eq $td->name ) &&
                         ( ref($_) eq 'Affix::Wrap::Enum' || ref($_) eq 'Affix::Wrap::Struct' )   &&
-                        ( abs( $_->end_line - $td->line ) <= 1 || abs( $_->end_line - $td->end_line ) <= 1 )
+                        ( abs( $_->end_line - $td->line ) <= 2 || abs( $_->end_line - $td->end_line ) <= 2 )
                 } @$objs;
                 if ($child) {
                     my $new = Affix::Wrap::Typedef->new(
@@ -948,6 +951,9 @@ package Affix::Wrap v1.0.7 {
                 if ( ( ref($node) eq 'Affix::Wrap::Struct' || ref($node) eq 'Affix::Wrap::Enum' ) &&
                     length( $node->name ) &&
                     $node->name ne '(anonymous)' ) {
+
+                    # If there's already a typedef for this name, skip it
+                    next if grep { $_ isa Affix::Wrap::Typedef && $_->name eq $node->name } @$objs;
                     my $new = Affix::Wrap::Typedef->new(
                         name         => $node->name,
                         underlying   => $node,
@@ -991,10 +997,9 @@ package Affix::Wrap v1.0.7 {
 
         method _is_valid_file ($f) {
             return 0 unless defined $f && length $f;
-            return 0 if $f =~ m{^/usr/(include|lib|share|local/include)};
-            return 0 if $f =~ m{^/System/Library};
-            return 0 if $f =~ m{(Program Files|Strawberry|MinGW|Windows|cygwin|msys)};
-            return 1;
+            return $f !~ m{^/usr/(include|lib|share|local/include)} &&
+                $f !~ m{^/System/Library} &&
+                $f !~ m{(Program Files|Strawberry|MinGW|Windows|cygwin|msys)}i;
         }
 
         method parse( $entry_point, $ids //= [] ) {
@@ -1126,13 +1131,13 @@ package Affix::Wrap v1.0.7 {
 
             # Global variables
             while ( $c
-                =~ /^\s*extern\s+((?:const\s+|unsigned\s+|struct\s+|[\w:<>]+(?:\s*::\s*[\w:<>]+)*\s*\*?\s*)+?)\s*(\w+(?:\[.*?\])?)\s*(?:=\s*[^;]+)?\s*;\s*(?:\/\/([^\r\n]*))?/gm
+                =~ /^\s*extern\s+((?:const\s+|unsigned\s+|struct\s+|[\w:<>]+(?:\s*::\s*[\w:<>]+)*\s*\*?\s*)+?)([\s\*]+)([a-zA-Z_]\w*(?:\[.*?\])?)\s*(?:=\s*[^;]+)?\s*;\s*(?:\/\/([^\r\n]*))?/gm
             ) {
                 my $s        = $-[0];
                 my $e        = $+[0];
-                my $type_str = $1;
-                my $name     = $2;
-                my $trail    = $3;
+                my $type_str = $1 . $2;
+                my $name     = $3;
+                my $trail    = $4;
 
                 # Strip likely API macros (uppercase words) from the type definition
                 $type_str =~ s/\b[A-Z_][A-Z0-9_]*\b//g;
@@ -1172,11 +1177,14 @@ package Affix::Wrap v1.0.7 {
                     my $ret = Affix::Wrap::Type->parse($ret_str);
                     my @args;
                     if ( defined $args_str && length $args_str && $args_str ne 'void' ) {
-                        my @args_raw = grep {length} map { s/^\s+|\s+$//g; $_ } split /,/, $args_str;
+                        my @args_raw = grep {length} map { s/^\s+|\s+$//g; $_ } split /,(?![^(]*\))/, $args_str;
                         if ( @args_raw == 1 && $args_raw[0] =~ /^void$/ ) { @args_raw = (); }
                         for my $raw (@args_raw) {
-                            if ( $raw =~ /^(.+?)\s+(\w+(?:\[.*?\])?)$/ ) {
-                                push @args, Affix::Wrap::Type->parse($1);
+                            if ( $raw =~ /^(.+?)([\s\*]+)([a-zA-Z_]\w*(?:\[.*?\])?)$/ ) {
+                                my ( $t, $sep, $n ) = ( $1, $2, $3 );
+                                $t .= $sep;
+                                if ( $n =~ s/(\[.*\])$// ) { $t .= $1 }
+                                push @args, Affix::Wrap::Type->parse($t);
                             }
                             else {
                                 push @args, Affix::Wrap::Type->parse($raw);
@@ -1196,8 +1204,10 @@ package Affix::Wrap v1.0.7 {
                         end_offset   => $e
                         );
                 }
-                elsif ( $content =~ /^(.+?)\s+(\w+)$/ ) {
-                    my ( $type_str, $name ) = ( $1, $2 );
+                elsif ( $content =~ /^(.+?)([\s\*]+)([a-zA-Z_]\w*(?:\[.*?\])?)$/ ) {
+                    my ( $type_str, $sep, $name ) = ( $1, $2, $3 );
+                    $type_str .= $sep;
+                    if ( $name =~ s/(\[.*\])$// ) { $type_str .= $1; }
                     push @$acc,
                         Affix::Wrap::Typedef->new(
                         name         => $name,
@@ -1266,8 +1276,9 @@ package Affix::Wrap v1.0.7 {
                         my $code_ref = Affix::Wrap::Type::CodeRef->new( ret => $ret, params => \@p );
                         push @args, Affix::Wrap::Argument->new( type => $code_ref, name => $cb_name );
                     }
-                    elsif ( $raw =~ /^(.+?)\s+(\w+(?:\[.*?\])?)$/ ) {
-                        my ( $t, $n ) = ( $1, $2 );
+                    elsif ( $raw =~ /^(.+?)([\s\*]+)([a-zA-Z_]\w*(?:\[.*?\])?)$/ ) {
+                        my ( $t, $sep, $n ) = ( $1, $2, $3 );
+                        $t .= $sep;
                         if ( $n =~ s/(\[.*\])$// ) { $t .= $1 }
                         push @args, Affix::Wrap::Argument->new( type => Affix::Wrap::Type->parse($t), name => $n );
                     }
@@ -1327,8 +1338,9 @@ package Affix::Wrap v1.0.7 {
                     $pending_doc = '';
                     next;
                 }
-                if ( $b =~ s/^\s*([\w\s\*]+?)\s+(\w+(?:\[.*?\])?)\s*;// ) {
-                    my ( $t, $n ) = ( $1, $2 );
+                if ( $b =~ s/^\s*(.+?)([\s\*]+)([a-zA-Z_]\w*(?:\[.*?\])?)\s*;// ) {
+                    my ( $t, $sep, $n ) = ( $1, $2, $3 );
+                    $t .= $sep;
                     $t =~ s/^\s+|\s+$//g;
                     if ( $n =~ s/(\[.*\])$// ) { $t .= $1 }
                     push @m, Affix::Wrap::Member->new( name => $n, type => Affix::Wrap::Type->parse($t), doc => $clean->($pending_doc) );
@@ -1391,26 +1403,11 @@ package Affix::Wrap v1.0.7 {
             $driver->parse( $entry_point, $include_dirs );
         }
 
-        method wrap ( $lib, $target //= [caller]->[0] ) {
-
-            # Pre-register User Types
-            # This ensures they are available in the Affix registry before signatures are parsed,
-            # and allows using them in recursive definitions or opaque handles.
-            for my $name ( keys %$types ) {
-                my $type     = $types->{$name};
-                my $type_str = builtin::blessed($type) ? $type : "$type";
-                Affix::typedef( $name, $type_str );
-            }
-            my @nodes = $self->parse;
-
-            #  Macro resolution pass
+        method _resolve_macros ($nodes) {
             my %macros;
-            for my $node (@nodes) {
+            for my $node (@$nodes) {
                 if ( $node isa Affix::Wrap::Macro ) {
                     my $val = $node->value // '';
-
-                    # Strip C suffixes (U, L, UL, LL) from hex/decimal numbers
-                    # e.g. 0x01U -> 0x01
                     $val =~ s/(?<=\d)[Uu][Ll]{0,2}//g;
                     $macros{ $node->name } = $val;
                 }
@@ -1452,6 +1449,57 @@ package Affix::Wrap v1.0.7 {
                 # Fallback: Treat as simple alias (A -> B)
                 return $cache{$token} = $resolve->($expr);
             };
+            for my $node (@$nodes) {
+                if ( $node isa Affix::Wrap::Macro ) {
+                    my $val = $resolve->( $node->name );
+                    if ( defined $val ) {
+                        $node->set_value($val);
+                    }
+                }
+            }
+        }
+
+        method generate ( $lib, $pkg, $file ) {
+            my @nodes = $self->parse;
+            $self->_resolve_macros( \@nodes );
+            my $out =<<~"";
+            package $pkg {
+                use v5.36;
+                use Affix;
+                #
+                my \$lib = '$lib';
+
+            for my $name ( keys %$types ) {
+                my $type     = $types->{$name};
+                my $type_str = builtin::blessed($type) ? $type : "'$type'";    # Quote user types
+                $out .= "typedef '$name' => $type_str;\n";
+            }
+            for my $node (@nodes) {
+                if ( ( $node isa Affix::Wrap::Typedef || $node isa Affix::Wrap::Struct || $node isa Affix::Wrap::Enum ) &&
+                    exists $types->{ $node->name } ) {
+                    next;
+                }
+                my $code = $node->affix_type;
+                if ($code) { $out .= "$code;\n"; }
+            }
+            $out .= "\n};\n1;\n";
+            Path::Tiny::path($file)->spew_utf8($out);
+        }
+
+        method wrap ( $lib, $target //= [caller]->[0] ) {
+
+            # Pre-register User Types
+            # This ensures they are available in the Affix registry before signatures are parsed,
+            # and allows using them in recursive definitions or opaque handles.
+            for my $name ( keys %$types ) {
+                my $type     = $types->{$name};
+                my $type_str = builtin::blessed($type) ? $type : "$type";
+                Affix::typedef( $name, $type_str );
+            }
+            my @nodes = $self->parse;
+
+            #  Macro resolution pass
+            $self->_resolve_macros( \@nodes );
 
             # Generation pass
             my @installed;
@@ -1462,41 +1510,15 @@ package Affix::Wrap v1.0.7 {
                     exists $types->{ $node->name } ) {
                     next;
                 }
-                if ( $node isa Affix::Wrap::Macro ) {
-
-                    # Attempt to resolve value to a pure number
-                    my $val = $resolve->( $node->name );
-                    if ( defined $val ) {
-
-                        # It's a number. Inject constant sub returning integer
-                        no strict 'refs';
-                        no warnings 'redefine';
-                        *{ "${target}::" . $node->name } = sub () {$val};
-                        push @installed, $node;
-                    }
-                    else {
-                        # It's complex or a string. Fallback to original behavior
-                        # This preserves definitions like: #define VER "1.2.3"
-                        $node->affix( undef, $target );
-                        push @installed, $node;
-                    }
-                }
-                elsif ( $node isa Affix::Wrap::Variable || $node isa Affix::Wrap::Function ) {
+                if ( $node->can('affix') ) {
                     $node->affix( $lib, $target );
                     push @installed, $node;
                 }
-                elsif ( $node->can('affix') ) {
-                    $node->affix( $lib, $target );
-                    push @installed, $node;
-                }
-                else {
-                    # use Data::Printer; p $node;
-                }
             }
             @installed;
         }
     }
-};
+}
 1;
 __END__
 Copyright (C) Sanko Robinson.
